{% extends "base.html" %}
{% load i18n %}

{% load quiz_tags %}

{% block title %} {{ quiz.title }} {% endblock %}
{% block description %} {{ quiz.title }} - {{ quiz.description }} {% endblock %}

{% block content %}


{% if user.is_authenticated %}
  <p>{% trans "You have already sat this exam and only one sitting is permitted" %}.</p>
{% else %}
  <p>{% trans "This exam is only accessible to signed in users" %}.</p>
{% endif %}

{% endblock %}
