{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Quizzes related to" %} {{ category.category }}{% endblock %}

{% block content %}
<h1>{% trans "Quizzes in the" %} <strong>{{ category.category }} </strong>{% trans "category" %}</h1>

  {% with object_list as quizzes %}
    {% if quizzes %}
        <ul>
        {% for quiz in quizzes %}
            <li>
			  <a href="{% url 'quiz_question' quiz_name=quiz.url %}">
				{{ quiz.title }}
			  </a>
			</li>
        {% endfor %}
        </ul>
    {% else %}
        <p>{% trans "There are no quizzes" %}</p>
    {% endif %}
  {% endwith %}
{% endblock %}
